Method: Metasploit::Credential::Login#last_attempted_at
- Defined in:
- app/models/metasploit/credential/login.rb
#last_attempted_at ⇒ DateTime
Note:
This is the last time this login was attempted and should be updated even if #status does not change. If
#status does not change, then normally #updated_at would be updated as the record would not save.
The last time a login was attempted.
|
|
# File 'app/models/metasploit/credential/login.rb', line 67
|