Class: Metasploit::Credential::Origin::Service
- Inherits:
-
ApplicationRecord
- Object
- ApplicationRecord
- Metasploit::Credential::Origin::Service
- Defined in:
- app/models/metasploit/credential/origin/service.rb
Overview
Origin of Metasploit::Credential::Cores that were gathered from a #service by a
auxiliary or exploit module. Contrast with Session which is
for Metasploit::Credential::Cores derived after a session is
gained and a post module is run from the session to
gather credentials.
Constant Summary collapse
- MODULE_FULL_NAME_REGEXP =
Regular expression that matches any
Mdm::Module::Detail#fullnamefor #module_full_name whereMdm::Module::Detail#mtypeis the module type, e.g.'auxiliary'or'exploit', and the remainder is a validMdm::Module::Detail#refname(it does not contain a'\'and is lower case alphanumeric). /\A(?<module_type>auxiliary|evasion|exploit|payload|post)\/(?<reference_name>[\-0-9A-Z_a-z]+(?:\/[\-0-9A-Z_a-z]+)*)\Z/
Instance Attribute Summary collapse
-
#cores ⇒ ActiveRecord::Relation<Metasploit::Credential::Core>
Core credentials imported from #service using the auxiliary or exploit module.
-
#module_full_name ⇒ String
The
Mdm::Module::Detail#fullnameof the auxiliary or exploit module that accessed #service. -
#service ⇒ Mdm::Service
The service from which the core credentials were gathered.
Instance Attribute Details
#cores ⇒ ActiveRecord::Relation<Metasploit::Credential::Core>
Core credentials imported from #service using the auxiliary or exploit module.
25 26 27 28 |
# File 'app/models/metasploit/credential/origin/service.rb', line 25 has_many :cores, as: :origin, class_name: 'Metasploit::Credential::Core', dependent: :destroy |
#module_full_name ⇒ String
The Mdm::Module::Detail#fullname of the auxiliary or exploit module that accessed #service.
|
|
# File 'app/models/metasploit/credential/origin/service.rb', line 42
|
#service ⇒ Mdm::Service
The service from which the core credentials were gathered.
34 35 36 |
# File 'app/models/metasploit/credential/origin/service.rb', line 34 belongs_to :service, class_name: 'Mdm::Service', inverse_of: :credential_origins |